/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jcr.taglib;

import com.squeakysand.commons.lang.StringUtils;
import com.squeakysand.jcr.NodeUtils;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;

import java.io.IOException;

import javax.jcr.Node;
import javax.jcr.Session;
import javax.servlet.jsp.JspException;

/**
 * Injects a JCR Node object into the page context based on the supplied path.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@JspTag
public class NodeTag extends JcrSimpleTagSupport {

    private String var;
    private String path;

    @Override
    public void doTag() throws JspException, IOException {
        Session session = getJcrSession();
        Node node = NodeUtils.getNodeByPath(path, session);
        if (node != null) {
            String determinedVar = determineVar();
            getJspContext().setAttribute(determinedVar, node);
        }
    }

    public String getPath() {
        return path;
    }

    /**
     * The fully qualified path to the Node to inject into the page context.
     * 
     * @param path
     *            fully qualified path.
     */
    @JspTagAttribute(required = true)
    public void setPath(String path) {
        this.path = path;
    }

    private String determineVar() {
        String result = null;
        if (StringUtils.isNullOrEmpty(var)) {
            result = AttributeNames.CURRENT_NODE;
        } else {
            result = var;
        }
        return result;
    }
}
