/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jcr.taglib;

import com.squeakysand.jsp.tagext.BodyContent;
import com.squeakysand.jsp.tagext.EnhancedSimpleTagSupport;
import com.squeakysand.jsp.tagext.VariableScope;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;
import com.squeakysand.jsp.tagext.annotations.JspTagVariable;

import java.io.IOException;

import javax.jcr.LoginException;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.servlet.jsp.JspException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@JspTag(bodyContent = BodyContent.SCRIPTLESS)
public class SessionTag extends EnhancedSimpleTagSupport {

    @JspTagVariable(variableClass = Session.class, scope = VariableScope.AT_END)
    private static final String VARIABLE_NAME = AttributeNames.CURRENT_SESSION;

    private static final Logger LOG = LoggerFactory.getLogger(SessionTag.class);

    private Repository repository;
    private Session session;

    @Override
    public void doTag() throws JspException, IOException {
        Repository determinedRepository = determineRepository();
        if (determinedRepository != null) {
            try {
                session = determinedRepository.login();
            } catch (LoginException e) {
                throw new JspException(e);
            } catch (RepositoryException e) {
                throw new JspException(e);
            }
        }
        if (session != null) {
            setPageAttribute(VARIABLE_NAME, session);
            doBody();
        } else {
            throw new JspException(new IllegalStateException("unable to determine current session"));
        }
    }

    @JspTagAttribute(rtexprvalue = true)
    public void setRepository(Repository repository) {
        this.repository = repository;
    }

    private Repository determineRepository() {
        Repository result = null;
        if (repository != null) {
            result = repository;
        } else {
            try {
                result = findAttribute(AttributeNames.CURRENT_REPOSITORY, Repository.class);
            } catch (ClassCastException e) {
                LOG.error(e.getMessage(), e);
            }
        }
        return result;
    }

    private void doBody() throws JspException, IOException {
        try {
            getJspBody().invoke(null);
        } finally {
            if (session != null) {
                session.logout();
                session = null;
            }
            removePageAttribute(AttributeNames.CURRENT_SESSION);
        }
    }
}
